# Author: Sam Green, Created: 18-10-2017
# Script sets up a set of core classes/functions to read data from silo data files.

# Comments from previous script version (Silo_Modules.py):
# -2016-06-10 SG: Opens MultiMesh silo files and can plot 2D structures.
# -2016-09-29 SG: PARAMETER and PARAMETER_MAIN Function added to save density and temperature data depending on
# what variable name is called,
# no longer separate functions to calculate density and temperature data.
# -2016-10-07 SG: PARAMETER Function modified to allow velocity to be chosen as the variable name.
# -2016-12.14 JM: self.Param and self.BigArray_Param no longer class
#   member data.
# -2017-02-23 SG: Added comments.
# -2017-03-10 SG: Added Class to read in VTK data produced by PION 2D projection code and calculate data to
# plot the column density and nebular line emission.

# New comments:
# - 2017-10-28 SG: Rewritten some functions to remove errors, increase efficiency.
# - 2017-10-29 SG: Bugs in reshaped_parameter2d function fixed.
# - 2018-03-26 SG: Added class to plot slices of the 3D data.
# - 22-04-2020 SG: Moved the parameter and variable function to SiloHeader_data.py
# - 22-04-2020 SG: Both functions now open all the files associated with each timestep and saves the data arrays for each of them.

# This is to make the scripts work with nested-grids.
# Works in 1D, 2D, and 3D. Should also work with non-nestedgrid too (i.e. 1 level of data).

# -------------- Set of libraries needed:
import numpy as np
import pypion
from pypion import SiloHeader_data


# --------------Class to access each 2D sub-domain and save density, temperature, etc. data:


class ReadData(SiloHeader_data.OpenData):

    # Function to read 1D data from multiple nested grid levels from
    # a Silo file, and return the data as numpy arrays with extents.
    def get_1Darray(self, param):

        level = self.nlevels()
        arr =  [[None]] * level
        level_max = [[None]] * level
        level_min = [[None]] * level
        sim_time = self.sim_time().value

        i = 0
        for file in self.data:
            self.open(i)

            variable_array = np.zeros((self.ngrid()[0]))

            domain=self.dom_size()
            a = domain['DomSize'][0]
            c = domain['Ndom'][0]
            e, min_ext = self.parameter(param)
            level_max[i] = self.level_max()
            level_min[i] = self.level_min()
            dx = (level_max[i][0]-level_min[i][0])/c

            for idom in range(c):
              # Sets the positions of each process array.
              ix = int((min_ext[idom][0] - level_min[i][0])*1.01/dx)
              x0 = ix * a
              x1 = x0 + a
              # Saves all the values into the 1D image array
              variable_array[x0:x1] = e[idom]

            arr[i] = variable_array
            i += 1

            del a
            del c
            del e
            del variable_array

        return {'data': arr, 'max_extents': level_max, 'min_extents': level_min, 'sim_time': sim_time}

    # Function to read 2D data from multiple nested grid levels from
    # a Silo file, and return the data as numpy arrays with extents.
    def get_2Darray(self, param):

        level = self.nlevels()
        arr =  [[None]] * level
        level_max = [[None]] * level
        level_min = [[None]] * level
        sim_time = self.sim_time().value

        i = 0
        for file in self.data:
            self.open(i)

            variable_array = np.zeros((self.ngrid()[1], self.ngrid()[0]))

            domain=self.dom_size()
            a = domain['DomSize'][0]
            b = domain['DomSize'][1]
            c = domain['Ndom'][0]
            d = domain['Ndom'][1]
            e, min_ext = self.parameter(param)
            level_max[i] = self.level_max()
            level_min[i] = self.level_min()
            dx = (level_max[i][0]-level_min[i][0])/c
            dy = (level_max[i][1]-level_min[i][1])/d
            ndom = c * d

            for idom in range(ndom):
              # get the (ix,iy,iz) location of sub-domain from its extents
              ix = int((min_ext[idom][0] - level_min[i][0])*1.01/dx)
              iy = int((min_ext[idom][1] - level_min[i][1])*1.01/dy)
              x0 = ix * a
              y0 = iy * b
              x1 = x0 + a
              y1 = y0 + b
              variable_array[y0:y1, x0:x1] = e[idom]
              
            arr[i] = variable_array
            i += 1

            del a
            del b
            del c
            del d
            del e
            del variable_array

        return {'data': arr, 'max_extents': level_max, 'min_extents': level_min, 'sim_time': sim_time}

    # Function to read 3D data from multiple nested grid levels from
    # a Silo file, and return the data as numpy arrays with extents.
    def get_3Darray(self, param):

        level = self.nlevels()
        arr =  [[None]] * level
        level_max = [[None]] * level
        level_min = [[None]] * level
        sim_time = self.sim_time()

        i = 0
        for file in self.data:
            self.open(i)

            variable_array = np.zeros((self.ngrid()[2], self.ngrid()[1], self.ngrid()[0]))
            domain=self.dom_size()
            a = domain['DomSize'][0]
            b = domain['DomSize'][1]
            f = domain['DomSize'][2]
            c = domain['Ndom'][0]
            d = domain['Ndom'][1]
            g = domain['Ndom'][2]
            e, min_ext = self.parameter(param)
            level_max[i] = self.level_max()
            level_min[i] = self.level_min()
            dx = (level_max[i][0]-level_min[i][0])/c
            dy = (level_max[i][1]-level_min[i][1])/d
            dz = (level_max[i][2]-level_min[i][2])/g
            ndom = c * d * g

            for idom in range(ndom):
              # get the (ix,iy,iz) location of sub-domain from its extents
              ix = int((min_ext[idom][0] - level_min[i][0])*1.01/dx)
              iy = int((min_ext[idom][1] - level_min[i][1])*1.01/dy)
              iz = int((min_ext[idom][2] - level_min[i][2])*1.01/dz)
              x0 = ix * a
              y0 = iy * b
              z0 = iz * f
              x1 = x0 + a
              y1 = y0 + b
              z1 = z0 + f
              variable_array[z0:z1, y0:y1, x0:x1] = e[idom]
              
            arr[i] = variable_array
            i += 1

            del a
            del b
            del c
            del d
            del e
            del g
            del f
            del variable_array

        return {'data': arr, 'max_extents': level_max, 'min_extents': level_min, 'sim_time': sim_time}



# Author: Sam Green, Created: 18-10-17
# Script opens up the header of silo files and saves some variables.

# New comments:
# - 2016-12.14 JM: ReadData has a CLOSE() function that calls the OpenSilo
#   CLOSE() function.  Xmin/Xmax/time etc. all now new directory to /header.
# - 15-02-2019 SG: Added new functions level_max and level_min and nlevels to help with nested grid python.
# - 22-04-2020 SG: Added all functions that use the init function in the OpenData class.
# - 22-04-2020 SG: Removed hard-coded cm in xmax and xmin.

# -------------- Set of libraries needed:
import sys

# Point to silo installation folder.
# I recommend installed silo here at ~/.silo/
sys.path.insert(0,"~/.local/silo/")

#sys.path.insert(0,"/home/pion_python/Library")
import Silo
import numpy as np
from astropy import units as u
from os import path as ospath

# -------------- Class to open Silo files and save variables.


class OpenData:
    # This will open the .SILO file and enters the 'header' directory.
    # If using multiple levels, this will open the level 0 file.
    def __init__(self, files):
        self.data = files
        self.db = Silo.Open(files[0])
        self.db.SetDir('/header')
        self.OpenFile = files[0]

    # Call this function with a for loop to cycle through each level file.
    def open(self, level):
        self.db.Close()
        self.db = Silo.Open(self.data[level])
        self.db.SetDir('/header')
        self.OpenFile = self.data[level]

    # To close all the variables after use.
    def close(self):
        self.db.Close()
        self.OpenFile = ""

    # Returns the header information:
    def header_info(self):
        header = self.db.GetToc()
        return header

    # Xmax variable contains the max value for every axis of the grid.
    def xmax(self):
        self.db.SetDir('/header')
        xmax = self.db.GetVar("Xmax")
        return xmax

    # Xmin variable contains the min value for every axis of the grid.
    def xmin(self):
        self.db.SetDir('/header')
        xmin = self.db.GetVar("Xmin")
        return xmin

    # level_xmax variable contains the max value for every axes of each level.
    def level_max(self):
        self.db.SetDir('/header')
        level_max = self.db.GetVar("level_xmax")
        return level_max

    # level_xmin variable contains the min value for every axis of each level.
    def level_min(self):
        self.db.SetDir('/header')
        level_min = self.db.GetVar("level_xmin")
        return level_min

    # nlevels variable contains the number of levels in the simulation.
    def nlevels(self):
        self.db.SetDir('/header')
        level = self.db.GetVar("grid_nlevels")
        return level

    # ndim variable: how many spatial dimensions on grid
    def ndim(self):
        self.db.SetDir('/header')
        dim = self.db.GetVar("gridndim")
        return dim

    # cycle is the simulation timestep on the finest level.
    def cycle(self):
        self.db.SetDir('/')
        cycle = self.db.GetVar("cycle")
        return cycle

    # sim_time variable contains the simulation time.
    def sim_time(self):
        self.db.SetDir('/header')
        simtime = self.db.GetVar("t_sim") * u.second
        return simtime

    # ngrid variable contains the size of the grid.
    def ngrid(self):
        self.db.SetDir('/header')
        ngrid = self.db.GetVar("NGrid")
        ngrid = [int(_) for _ in ngrid]
        return ngrid

    # nproc variable contains the number of domains in the silo file.
    def nproc(self):
        self.db.SetDir('/header')
        nproc = self.db.GetVar("MPI_nproc")
        return nproc

    # dom_size contains the size of the domain in the grid.
    def dom_size(self):
        ndom = np.array([1, 1, 1])
        cells = np.copy(self.ngrid())
        i = 1
        while i < self.nproc():
            axis = np.argmax(cells)
            i *= 2
            ndom[axis] *= 2
            cells[axis] /= 2
        domsize = self.ngrid() / ndom
        domsize = [int(_) for _ in domsize]
        ndom1 = ndom
        ndom1 = [int(_) for _ in ndom1]
        return {'DomSize': domsize, 'Ndom': ndom1}

    # Retrieves the requested data from the silo file
    def variable(self, par):
        # Saves the selected data as a variable.
        param = self.db.GetVar(par + "_data")
        # Saves the selected data's dimensions.
        param_dims = self.db.GetVar(par + "_dims")
        # Reshapes the dimensions into the correct orientation.
        if (self.ndim()==1):
          param_dims = [param_dims]
        else:
          param_dims = param_dims[::-1]
        # Puts the array into the correct format, i.e. (a,b).
        param = np.array(param).reshape(param_dims)
        # return the data, and the number of dimensions
        return param, len(param_dims)

    # Opens up the sub-domains and saves the data specified in "variable".
    # "data" is a string corresponding to a scalar variable, e.g. "Density"
    def parameter(self, data):
        array_param = []  # data for each sub-domain
        exten_param = []  # min-extents of each sub-domain
        self.db.SetDir("/")
        extents = self.db.GetVar("MultiMesh_extents")
        i = 0  # index in extents
        pp = self.db.GetVar(data+"_varnames")
        newfile=""
        nowfile=""
        newpar=""
        origfile=""
        #for n in paths:
        for n in pp:
            if n=="":
              continue # 1st and last elements are empty
            # if "n" contains a ":" then the domain is in a different file.
            # We need to close the current file and open another one.
            loc=n.find(":")
            if loc != -1:
              if (newfile==""):
                origfile=self.OpenFile
                nowfile=self.OpenFile
              else:
                nowfile=newfile
              # split newfile into filename + location
              newfile= n[:loc]
              newpar = n[loc+1:]
              # append full path to new filename
              newfile = ospath.dirname(self.OpenFile) + "/" + newfile
              # only close old / open new file if they are not the same file
              if (newfile != nowfile):
                self.db.Close()
                self.db = Silo.Open(newfile)
                self.db.SetDir("/")
                nowfile=newfile
            else:
              # nothing do do because domain is in my file.
              newpar = n
              nowfile= self.OpenFile
            #print(nowfile,newpar)
            d,v = ospath.split(newpar)
            rank = d[6:10]  # save rank so i can get extents.
            self.db.SetDir(d)
            variable_data, idim = self.variable(v)
            array_param.append(variable_data)
            min_extents = extents[i:i+idim]
            i = i + 2*idim
            exten_param.append(min_extents)
        # re-open original file that was being used for the 1st sub-domain.
        self.db.Close()
        self.db = Silo.Open(self.OpenFile)
        self.db.SetDir("/")
        return np.array(array_param), np.array(exten_param)




# Author: Sam Green, Created: 11-10-20

# Script to call all PyPion classes to plot 2d + 3d Silo data. Works for uniform and nested-grid data.

import pypion
from pypion import Plotting_Classes
from pypion import argparse_command
#-------------------------------
import matplotlib
# Using this to stop matplotlib from using a $DISPLAY environment variable.
# i.e. This now works over ssh without the need for a x-server.
matplotlib.use('Agg')
import matplotlib.pyplot as plt
#-------------------------------
# The way I plot 3 arrays onto the same grid throws up a warning.
# This just surpresses the warning.
import warnings
warnings.filterwarnings("ignore", category=matplotlib.cbook.mplDeprecation)
#-------------------------------
line = argparse_command.InputValues()
time_dicts = line.time_dicts
dimen = line.dimen
#-------------------------------

for files in time_dicts:

	arr = time_dicts[files]

	# Change var1 to modify specific settings for your plot.
	# var1 = ["parameter", max, min, "colour scale", "axis(y or z)", ngrid/2 - 1 (so for 128^3 this is 63, 256^3 it's 127, etc)]
	var1 = ["Density", -22, -27, "viridis", 'y', 63]
	# var1 = ["Temperature", 8, 3, "inferno", 'log', 'y', 127]

	fig = plt.figure()

	# To plot 2D data use the following line to plot the data:
	#a = Plotting2d(arr).plotsilo_2d(var1[0], fig, var1)
	
	# To plot 3D data use the following line to plot the data:
	a = Plotting_Classes.Plotting3d(arr).XZXYslice(var1[0], fig, var1)

	imagefile = "%s%s_%s.png" % (line.img_path, line.img_file, time_dicts[files][0][len(time_dicts[files][0]) - 13:len(time_dicts[files][0]) - 5])
	plt.savefig(imagefile, bbox_inches='tight', dpi=300)




# Author: Sam Green, Created: 20-10-17
# Script that defines functions to use the data created from core2D_Silo.py to plot the silo file's data.

# Comments from previous script version (Silo_Plot.py):
# -2016-09-22 SG: Created to work with the Bubble Nebula simulations, adapted from MultiMesh_Silo_plot.py.
# -2016-09-23 JM/SG: Correction to fix the "too many files open" bug; edited to accommodate new InputValues class.
# -2016-12-14 JM: Set up instance of class "ThisFile" to try to fix
#   memory leak problems.  Also only setup InputValues() once.
# -2017-01-17 SG: Set density and temperature to a constant max and min.
# -2017-5-22 SG: Few small changes made to fix some bugs.

# New comments:
# -2017-10-28 SG: Set up plotting code as a function
# -2017-11-6 SG: New class to plot velocity data as vector/streamline
# -2017-12-05 SG: Few changes made to temp plotting function.
# -2019-07-22 SG: Added class to plot slices of the 3D data.
# -2019-08-12 SG: Tidied up a few things.

#############################################################
# Here's what each function does at a glance:

# 3D stuff:
# XZslice - plots a slice in the XZ-direction.
# XYslice - plots a slice in the XY-direction.
# XZXYslice - plots a slice in the XZ and XY -direction onto the same figure.

# magplot - plots the magnitude of the magnetic field.
#----------------------------------------------------------
# 2D stuff:
# plot_single - plots a single parameter in the top-half of the sim.
# plot_double - plots 1 parameter above the x-axis and another below it.
# Used when you've anly simulated half on the 2D object.

#############################################################
# -------------- Set of libraries needed:
import pypion
from pypion import ReadData

import matplotlib
# Using this to stop matplotlib from using a $DISPLAY environment variable.
# i.e. This now works over ssh.
#matplotlib.use('Agg')
from matplotlib.colorbar import Colorbar
import matplotlib.pyplot as plt
import matplotlib as mpl
from mpl_toolkits.axes_grid1 import make_axes_locatable
from matplotlib.ticker import MultipleLocator
import matplotlib.gridspec as gridspec

import numpy as np
from astropy import units as u

import warnings
warnings.filterwarnings("ignore", category=matplotlib.cbook.mplDeprecation)

plt.rc('font', **{'size': 12})
# plt.rc('lines', linewidth=2)
plt.rc('font', weight='bold')  # <-------------

# -------------- Class to plot the data from 2d Silo files.

class Plotting2d(ReadData.ReadData):
    def plotsilo_2d(self, lim_min, lim_max, fig, var1, var2):
        xmin = (self.xmin() * u.cm).to(u.pc)  # Set data to units of parsecs.
        xmax = (self.xmax() * u.cm).to(u.pc)

        var1 = var1
        var2 = var2

        level_min = (self.level_min() * u.cm).to(u.pc)  # Set data to units of parsecs.
        level_max = (self.level_max() * u.cm).to(u.pc)

        ax1 = fig.add_subplot(2, 1, 1)
        ax1.set_title('      Time = %5.5f Myr' % self.sim_time().value)

        if var1[4] == 'log':
            log_d = np.log10(self.get_2Darray(var1[0])['data'])
        else:
            log_d = self.get_2Darray(var1[0])['data']

        ax1.set_xlim(lim_min[0].value, lim_max[0].value)
        ax1.set_ylim(lim_min[1].value, lim_max[1].value)

        im1 = ax1.imshow(log_d, interpolation='nearest', cmap=var1[3],
                         extent=[level_min[0].value, level_max[0].value, level_min[1].value,
                                 level_max[1].value],
                         origin='lower', vmax=var1[1], vmin=var1[2])
        divider1 = make_axes_locatable(ax1)  # Create divider for existing axes instance.
        cax1 = divider1.append_axes("right", size="5%", pad=0.05)  # Append axes to the right of ax1.
        cbar1 = plt.colorbar(im1, cax=cax1, ticks=MultipleLocator(1),
                             format="%.2f")  # Create colorbar in the appended axes.
        txt1 = ax1.text(0.8, 0.92, r'$log(\rho)$', transform=ax1.transAxes)
        ax1.axes.get_xaxis().set_visible(False)  # Remove the x-axis.

        # Temperature plot:
        ax2 = fig.add_subplot(2, 1, 2)

        if var1[4] == 'log':
            log_t = np.log10(self.get_2Darray(var2[0])['data'])
        else:
            log_t = self.get_2Darray(var2[0])['data']

        ax2.set_xlim(lim_min[0].value, lim_max[0].value)
        ax2.set_ylim(-lim_max[1].value, lim_min[1].value)

        im2 = ax2.imshow(log_t, interpolation='nearest', cmap=var2[3],
                         extent=[level_min[0].value, level_max[0].value,
                                 -level_max[1].value, -level_min[1].value],
                         vmax=var2[1], vmin=var2[2])
        divider2 = make_axes_locatable(ax2)
        cax2 = divider2.append_axes("right", size="5%", pad=0.05)
        cbar2 = plt.colorbar(im2, cax=cax2, ticks=MultipleLocator(1), format="%.2f")
        txt2 = ax2.text(0.8, 0.05, r'$log(T)$', transform=ax2.transAxes, color='white')
        ax2.set_xlabel('     x-axis (pc)')
        ax2.set_ylabel('                                              z-axis (pc)')

        fig.subplots_adjust(wspace=0, hspace=0)  # Remove the whitespace between the images

        del xmin
        del xmax
        # del var1
        # del var2
        # del level_min
        # del level_max
        # del log_d
        # del log_t
        # del im1
        # del im2
        # self.db.close()
        # del self.db

        return fig


    def  plot2d_1(self, param, Fig, var1):

        # assign necessary data to arrays.
        data = self.get_2Darray(param)['data']

        lim_max = (self.get_2Darray(param)['max_extents'] * u.cm).to(u.pc)
        lim_min = (self.get_2Darray(param)['min_extents'] * u.cm).to(u.pc)
        # sim_time = self.get_2Darray(param)['sim_time'].to(u.Myr)

        fig = Fig

        var1 = var1

        for i in range(len(data)):
            x = data[i]
            log_data = np.log10(x)

            # --------------Left Plot----------------------------

            ax1 = fig.add_subplot(1,1,1)

            # ax1.set_title('     Time = %5.5f Myr' % sim_time.value)

            ax1.set_xlim(lim_min[0][0].value, lim_max[0][0].value)
            ax1.set_ylim(lim_min[0][1].value, lim_max[0][1].value)

            im1 = ax1.imshow(log_data, interpolation='nearest', cmap="viridis",
                            extent=[lim_min[i][0].value, lim_max[i][0].value, lim_min[i][1].value, lim_max[i][1].value], origin='lower', vmax=-22, vmin=-27)

            ax1.set_xlabel('x-axis (pc)')
            ax1.set_ylabel('y-axis (pc)')

            #cbax = plt.subplot(gs[-1, 0:])
            #cb = Colorbar(ax=cbax, mappable=im1, orientation='horizontal', ticklocation='bottom')

        return fig

class Plotting3d(ReadData.ReadData):
    def XZXYslice(self, param, Fig, var1):

        # assign necessary data to arrays.
        data = self.get_3Darray(param)['data']
        lim_max = (self.get_3Darray(param)['max_extents'] * u.cm).to(u.pc)
        lim_min = (self.get_3Darray(param)['min_extents'] * u.cm).to(u.pc)
        sim_time = self.get_3Darray(param)['sim_time'].to(u.Myr)

        fig = Fig
        gs = gridspec.GridSpec(2, 2, height_ratios=[1, 0.05], width_ratios=[1, 1])
        gs.update(left=0.05, right=0.95, bottom=0.08, top=0.93, wspace=0.02, hspace=0.03)

        var = var1

        for i in range(len(data)):
            x_slice = data[i][var[5], :, :]
            y_slice = data[i][:, var[5], :]

            log_dx = np.log10(x_slice)
            log_dy = np.log10(y_slice)

            # --------------Left Plot----------------------------

            ax1 = fig.add_subplot(gs[0, 0])

            ax1.set_title('                                                 Time = %5.5f Myr' % sim_time.value)

            ax1.set_xlim(lim_min[0][0].value, lim_max[0][0].value)
            ax1.set_ylim(lim_min[0][1].value, lim_max[0][1].value)

            im1 = ax1.imshow(log_dy, interpolation='nearest', cmap="viridis",
                            extent=[lim_min[i][0].value, lim_max[i][0].value, lim_min[i][1].value, lim_max[i][1].value],
                            origin='lower', vmax=-22, vmin=-27)
            # txt1 = ax1.text(0.8, 0.92, r'$log(\rho)$', transform=ax1.transAxes)
            ax1.set_xlabel('x-axis (pc)')
            ax1.set_ylabel('z-axis (pc)')

            # --------------Right Plot----------------------------
            ax2 = fig.add_subplot(gs[0, 1])
            # ax2.set_title('Time = %5.5f Myr' % self.sim_time().value)

            ax2.set_xlim(lim_min[0][0].value, lim_max[0][0].value)
            ax2.set_ylim(lim_min[0][2].value, lim_max[0][2].value)

            im2 = ax2.imshow(log_dx, interpolation='nearest', cmap=var[3],
                            extent=[lim_min[i][0].value, lim_max[i][0].value, lim_min[i][2].value, lim_max[i][2].value],
                            origin='lower', vmax=var[1], vmin=var[2])
            # txt2 = ax2.text(0.8, 0.92, r'$log(\rho)$', transform=ax1.transAxes)
            ax2.set_xlabel('x-axis (pc)')
            ax2.yaxis.set_label_position("right")
            ax2.yaxis.tick_right()
            ax2.set_ylabel('y-axis (pc)')

            cbax = plt.subplot(gs[-1, 0:])
            cb = Colorbar(ax=cbax, mappable=im1, orientation='horizontal', ticklocation='bottom')
            # cb.set_label(r'Colorbar !', labelpad=10)

        #plt.savefig("test.png", bbox_inches='tight', dpi=300)

        return fig




#!/usr/bin/env python

from setuptools import setup, __version__
from setuptools.command.install import install
from pkg_resources import parse_version
import subprocess
import sys
import os

class CustomInstall(install):
    def run(self):
        command = "mkdir ~/.silo_test"
        process = subprocess.Popen(command, shell=True)
        process.wait()
        install.run(self)

minimum_version = parse_version('30.4.0')

if parse_version(__version__) < minimum_version:
    raise RuntimeError("Package setuptools must be at least version {}".format(minimum_version))

setup(cmdclass={'install': CustomInstall})

#os.system('mkdir ~/.silo_test')



# Author: Sam Green, Created: 18-10-2017
# Script to allow the user to use commandline arguments when running scripts.

# Comments from previous script version (Silo_Modules.py):
# - 2016.06.15 JM: sorting of list, adding path to filenames,
#   relative position for annotation.  Also added command-line args.
# - 2016.06.16 JM: improved removal of non-requested files.

# New Comments:
# - 15-02-2019 SG: Added a new argument to choose the type of file being used. Should've added it over a year ago.
# - 15-02-2019 SG: Added method to group files by timestep (useful for nested grid sims).

# --------------set of libraries needed:
import argparse
from os import listdir
import sys
sys.path.append('.')


class InputValues:
    def __init__(self):
        parser = argparse.ArgumentParser(
            description='Process some values.',
            usage='script.py <path-to-files> <base-filename> <image-path> <image-filename> <dimension>')
        parser.add_argument('Path', help='path to Silo files')
        parser.add_argument('file_base', help='base filename of silo files')
        parser.add_argument('img_path', help='path to save images into')
        parser.add_argument('img_base', help='base filename of image files')
        parser.add_argument('dimension', help='dimension of data (1d, 2d, 3d)')
        args = parser.parse_args()

        self.dimen = args.dimension

        # make sure that path has a trailing /
        file_path = args.Path
        if not file_path.endswith('/'):
            file_path += "/"

        self.img_path = args.img_path
        if not self.img_path.endswith('/'):
            self.img_path += "/"

        self.img_file = args.img_base

        # Reads all the files in a directory and then puts their names into a 1D array.
        data_files = [f for f in listdir(file_path)]  # if isfile(join(file_path, f))]

        # Sort list by timestep
        data_files.sort()

        search = ".silo"

        # Remove non-silo files, and non-requested files.
        # search = None
        # if args.file_type == 'silo':
        #    search = ".silo"
        # elif args.file_type == 'vtk':
        #    search = ".vtk"
        # elif args.file_type == 'fits':
        #    search = ".fits"

        data_files = [f for f in data_files if search in f]

        search = args.file_base
        data_files = [f for f in data_files if search in f]

        # Want to include only the primary data files, with "_0000.*.silo"
        #print(len(data_files))
        search = "_0001."
        data_files = [f for f in data_files if search not in f]
        search = "_0002."
        data_files = [f for f in data_files if search not in f]
        search = "_0003."
        data_files = [f for f in data_files if search not in f]
        search = "_0004."
        data_files = [f for f in data_files if search not in f]
        #print(len(data_files))

        # Add path to files
        self.data_files_with_path = [f.replace(f, file_path+f) for f in data_files]

        #  Establish dictionaries to group files by timestep (i.e. such that all layers in a given timestep are in
        #  a list assigned to a key with that timestep)
        self.time_dicts = {}
        for index in range(len(self.data_files_with_path)):

            filename = self.data_files_with_path[index]
            time = filename[len(filename) - 13: len(filename) - 5]
            try:
                self.time_dicts[time].append(filename)
            except:
                self.time_dicts[time] = [filename]
